@@ -2,23 +2,24 @@
 # Coldcard Electrum plugin main code.
 #
 #
-from struct import pack, unpack
-import os, sys, time, io
+import os, time, io
 import traceback
+from typing import TYPE_CHECKING, Optional
+import struct
 
+from electrum import bip32
 from electrum.bip32 import BIP32Node, InvalidMasterKeyVersionBytes
 from electrum.i18n import _
-from electrum.plugin import Device
-from electrum.keystore import Hardware_KeyStore, xpubkey_to_pubkey, Xpub
-from electrum.transaction import Transaction
-from electrum.wallet import Standard_Wallet
-from electrum.crypto import hash_160
+from electrum.plugin import Device, hook, runs_in_hwd_thread
+from electrum.keystore import Hardware_KeyStore, KeyStoreWithMPK
+from electrum.transaction import PartialTransaction
+from electrum.wallet import Standard_Wallet, Multisig_Wallet, Abstract_Wallet
 from electrum.util import bfh, bh2u, versiontuple, UserFacingException
 from electrum.base_wizard import ScriptTypeNotSupported
 from electrum.logging import get_logger
 
-from ..hw_wallet import HW_PluginBase
-from ..hw_wallet.plugin import LibraryFoundButUnusable
+from ..hw_wallet import HW_PluginBase, HardwareClientBase
+from ..hw_wallet.plugin import LibraryFoundButUnusable, only_hook_if_libraries_available
 
 
 _logger = get_logger(__name__)
@@ -30,10 +31,6 @@ try:
     from ckcc.protocol import CCProtoError, CCUserRefused, CCBusyError
     from ckcc.constants import (MAX_MSG_LEN, MAX_BLK_LEN, MSG_SIGNING_MAX_LENGTH, MAX_TXN_LEN,
         AF_CLASSIC, AF_P2SH, AF_P2WPKH, AF_P2WSH, AF_P2WPKH_P2SH, AF_P2WSH_P2SH)
-    from ckcc.constants import (
-        PSBT_GLOBAL_UNSIGNED_TX, PSBT_IN_NON_WITNESS_UTXO, PSBT_IN_WITNESS_UTXO,
-        PSBT_IN_SIGHASH_TYPE, PSBT_IN_REDEEM_SCRIPT, PSBT_IN_WITNESS_SCRIPT,
-        PSBT_IN_BIP32_DERIVATION, PSBT_OUT_BIP32_DERIVATION, PSBT_OUT_REDEEM_SCRIPT)
 
     from ckcc.client import ColdcardDevice, COINKITE_VID, CKCC_PID, CKCC_SIMULATOR_PATH
 
@@ -60,31 +57,11 @@ except ImportError:
 
 CKCC_SIMULATED_PID = CKCC_PID ^ 0x55aa
 
-def my_var_int(l):
-    # Bitcoin serialization of integers... directly into binary!
-    if l < 253:
-        return pack("B", l)
-    elif l < 0x10000:
-        return pack("<BH", 253, l)
-    elif l < 0x100000000:
-        return pack("<BI", 254, l)
-    else:
-        return pack("<BQ", 255, l)
 
-def xfp_from_xpub(xpub):
-    # sometime we need to BIP32 fingerprint value: 4 bytes of ripemd(sha256(pubkey))
-    # UNTESTED
-    kk = bfh(Xpub.get_pubkey_from_xpub(xpub, []))
-    assert len(kk) == 33
-    xfp, = unpack('<I', hash_160(kk)[0:4])
-    return xfp
+class CKCCClient(HardwareClientBase):
 
-
-class CKCCClient:
-    # Challenge: I haven't found anywhere that defines a base class for this 'client',
-    # nor an API (interface) to be met. Winging it. Gets called from lib/plugins.py mostly?
-
-    def __init__(self, plugin, handler, dev_path, is_simulator=False):
+    def __init__(self, plugin, handler, dev_path, *, is_simulator=False):
+        HardwareClientBase.__init__(self, plugin=plugin)
         self.device = plugin.device
         self.handler = handler
 
@@ -95,7 +72,6 @@ class CKCCClient:
             self.dev = ElectrumColdcardDevice(dev_path, encrypt=True)
         else:
             # open the real HID device
-            import hid
             hd = hid.device(path=dev_path)
             hd.open_path(dev_path)
 
@@ -105,24 +81,28 @@ class CKCCClient:
         # should expect. It's also kinda slow.
 
     def __repr__(self):
-        return '<CKCCClient: xfp=%08x label=%r>' % (self.dev.master_fingerprint,
+        return '<CKCCClient: xfp=%s label=%r>' % (xfp2str(self.dev.master_fingerprint),
                                                         self.label())
 
-    def verify_connection(self, expected_xfp, expected_xpub):
+    @runs_in_hwd_thread
+    def verify_connection(self, expected_xfp: int, expected_xpub=None):
         ex = (expected_xfp, expected_xpub)
 
         if self._expected_device == ex:
             # all is as expected
             return
 
+        if expected_xpub is None:
+            expected_xpub = self.dev.master_xpub
+
         if ( (self._expected_device is not None) 
                 or (self.dev.master_fingerprint != expected_xfp)
                 or (self.dev.master_xpub != expected_xpub)):
             # probably indicating programing error, not hacking
             _logger.info(f"xpubs. reported by device: {self.dev.master_xpub}. "
                          f"stored in file: {expected_xpub}")
-            raise RuntimeError("Expecting 0x%08x but that's not what's connected?!" %
-                               expected_xfp)
+            raise RuntimeError("Expecting %s but that's not what's connected?!" %
+                               xfp2str(expected_xfp))
 
         # check signature over session key
         # - mitm might have lied about xfp and xpub up to here
@@ -132,16 +112,16 @@ class CKCCClient:
 
         self._expected_device = ex
 
+        if not getattr(self, 'ckcc_xpub', None):
+            self.ckcc_xpub = expected_xpub
+
         _logger.info("Successfully verified against MiTM")
 
     def is_pairable(self):
-        # can't do anything w/ devices that aren't setup (but not normally reachable)
+        # can't do anything w/ devices that aren't setup (this code not normally reachable)
         return bool(self.dev.master_xpub)
 
-    def timeout(self, cutoff):
-        # nothing to do?
-        pass
-
+    @runs_in_hwd_thread
     def close(self):
         # close the HID device (so can be reused)
         self.dev.close()
@@ -155,12 +135,12 @@ class CKCCClient:
         # not be encrypted, so better for privacy if based on xpub/fingerprint rather than
         # USB serial number.
         if self.dev.is_simulator:
-            lab = 'Coldcard Simulator 0x%08x' % self.dev.master_fingerprint
+            lab = 'Coldcard Simulator ' + xfp2str(self.dev.master_fingerprint)
         elif not self.dev.master_fingerprint:
             # failback; not expected
             lab = 'Coldcard #' + self.dev.serial
         else:
-            lab = 'Coldcard 0x%08x' % self.dev.master_fingerprint
+            lab = 'Coldcard ' + xfp2str(self.dev.master_fingerprint)
 
         # Hack zone: during initial setup I need the xfp and master xpub but 
         # very few objects are passed between the various steps of base_wizard.
@@ -176,6 +156,7 @@ class CKCCClient:
 
         return LabelStr(lab, self.dev.master_fingerprint, self.dev.master_xpub)
 
+    @runs_in_hwd_thread
     def has_usable_connection_with_device(self):
         # Do end-to-end ping test
         try:
@@ -184,6 +165,7 @@ class CKCCClient:
         except:
             return False
 
+    @runs_in_hwd_thread
     def get_xpub(self, bip32_path, xtype):
         assert xtype in ColdcardPlugin.SUPPORTED_XTYPES
         _logger.info('Derive xtype = %r' % xtype)
@@ -199,6 +181,7 @@ class CKCCClient:
             xpub = node._replace(xtype=xtype).to_xpub()
         return xpub
 
+    @runs_in_hwd_thread
     def ping_check(self):
         # check connection is working
         assert self.dev.session_key, 'not encrypted?'
@@ -209,23 +192,33 @@ class CKCCClient:
         except:
             raise RuntimeError("Communication trouble with Coldcard")
 
+    @runs_in_hwd_thread
     def show_address(self, path, addr_fmt):
-        # prompt user w/ addres, also returns it immediately.
+        # prompt user w/ address, also returns it immediately.
         return self.dev.send_recv(CCProtocolPacker.show_address(path, addr_fmt), timeout=None)
 
+    @runs_in_hwd_thread
+    def show_p2sh_address(self, *args, **kws):
+        # prompt user w/ p2sh address, also returns it immediately.
+        return self.dev.send_recv(CCProtocolPacker.show_p2sh_address(*args, **kws), timeout=None)
+
+    @runs_in_hwd_thread
     def get_version(self):
         # gives list of strings
         return self.dev.send_recv(CCProtocolPacker.version(), timeout=1000).split('\n')
 
+    @runs_in_hwd_thread
     def sign_message_start(self, path, msg):
         # this starts the UX experience.
         self.dev.send_recv(CCProtocolPacker.sign_message(msg, path), timeout=None)
 
+    @runs_in_hwd_thread
     def sign_message_poll(self):
         # poll device... if user has approved, will get tuple: (addr, sig) else None
         return self.dev.send_recv(CCProtocolPacker.get_signed_msg(), timeout=None)
 
-    def sign_transaction_start(self, raw_psbt, finalize=True):
+    @runs_in_hwd_thread
+    def sign_transaction_start(self, raw_psbt: bytes, *, finalize: bool = False):
         # Multiple steps to sign:
         # - upload binary
         # - start signing UX
@@ -240,10 +233,12 @@ class CKCCClient:
         if resp != None:
             raise ValueError(resp)
 
+    @runs_in_hwd_thread
     def sign_transaction_poll(self):
         # poll device... if user has approved, will get tuple: (legnth, checksum) else None
         return self.dev.send_recv(CCProtocolPacker.get_signed_txn(), timeout=None)
 
+    @runs_in_hwd_thread
     def download_file(self, length, checksum, file_number=1):
         # get a file
         return self.dev.download_file(length, checksum, file_number=file_number)
@@ -254,6 +249,8 @@ class Coldcard_KeyStore(Hardware_KeyStore):
     hw_type = 'coldcard'
     device = 'Coldcard'
 
+    plugin: 'ColdcardPlugin'
+
     def __init__(self, d):
         Hardware_KeyStore.__init__(self, d)
         # Errors and other user interaction is done through the wallet's
@@ -262,41 +259,31 @@ class Coldcard_KeyStore(Hardware_KeyStore):
         self.force_watching_only = False
         self.ux_busy = False
 
-        # Seems like only the derivation path and resulting **derived** xpub is stored in
-        # the wallet file... however, we need to know at least the fingerprint of the master
-        # xpub to verify against MiTM, and also so we can put the right value into the subkey paths
-        # of PSBT files that might be generated offline. 
-        # - save the fingerprint of the master xpub, as "xfp"
-        # - it's a LE32 int, but hex more natural way to see it
+        # we need to know at least the fingerprint of the master xpub to verify against MiTM
         # - device reports these value during encryption setup process
+        # - full xpub value now optional
         lab = d['label']
-        if hasattr(lab, 'xfp'):
-            # initial setup
-            self.ckcc_xfp = lab.xfp
-            self.ckcc_xpub = lab.xpub
-        else:
-            # wallet load: fatal if missing, we need them!
-            self.ckcc_xfp = d['ckcc_xfp']
-            self.ckcc_xpub = d['ckcc_xpub']
+        self.ckcc_xpub = getattr(lab, 'xpub', None) or d.get('ckcc_xpub', None)
 
     def dump(self):
         # our additions to the stored data about keystore -- only during creation?
         d = Hardware_KeyStore.dump(self)
-
-        d['ckcc_xfp'] = self.ckcc_xfp
         d['ckcc_xpub'] = self.ckcc_xpub
-
         return d
 
-    def get_derivation(self):
-        return self.derivation
+    def get_xfp_int(self) -> int:
+        xfp = self.get_root_fingerprint()
+        assert xfp is not None
+        return xfp_int_from_xfp_bytes(bfh(xfp))
 
     def get_client(self):
         # called when user tries to do something like view address, sign somthing.
         # - not called during probing/setup
+        # - will fail if indicated device can't produce the xpub (at derivation) expected
         rv = self.plugin.get_client(self)
         if rv:
-            rv.verify_connection(self.ckcc_xfp, self.ckcc_xpub)
+            xfp_int = self.get_xfp_int()
+            rv.verify_connection(xfp_int, self.ckcc_xpub)
 
         return rv
 
@@ -337,8 +324,7 @@ class Coldcard_KeyStore(Hardware_KeyStore):
                                             % MSG_SIGNING_MAX_LENGTH)
             return b''
 
-        client = self.get_client()
-        path = self.get_derivation() + ("/%d/%d" % sequence)
+        path = self.get_derivation_prefix() + ("/%d/%d" % sequence)
         try:
             cl = self.get_client()
             try:
@@ -377,162 +363,24 @@ class Coldcard_KeyStore(Hardware_KeyStore):
         # give empty bytes for error cases; it seems to clear the old signature box
         return b''
 
-    def build_psbt(self, tx: Transaction, wallet=None, xfp=None):
-        # Render a PSBT file, for upload to Coldcard.
-        # 
-        if xfp is None:
-            # need fingerprint of MASTER xpub, not the derived key
-            xfp = self.ckcc_xfp
-
-        inputs = tx.inputs()
-
-        if 'prev_tx' not in inputs[0]:
-            # fetch info about inputs, if needed?
-            # - needed during export PSBT flow, not normal online signing
-            assert wallet, 'need wallet reference'
-            wallet.add_hw_info(tx)
-
-        # wallet.add_hw_info installs this attr
-        assert tx.output_info is not None, 'need data about outputs'
-
-        # Build map of pubkey needed as derivation from master, in PSBT binary format
-        # 1) binary version of the common subpath for all keys
-        #       m/ => fingerprint LE32
-        #       a/b/c => ints
-        base_path = pack('<I', xfp)
-        for x in self.get_derivation()[2:].split('/'):
-            if x.endswith("'"):
-                x = int(x[:-1]) | 0x80000000
-            else:
-                x = int(x)
-            base_path += pack('<I', x)
-
-        # 2) all used keys in transaction
-        subkeys = {}
-        derivations = self.get_tx_derivations(tx)
-        for xpubkey in derivations:
-            pubkey = xpubkey_to_pubkey(xpubkey)
-
-            # assuming depth two, non-harded: change + index
-            aa, bb = derivations[xpubkey]
-            assert 0 <= aa < 0x80000000
-            assert 0 <= bb < 0x80000000
-
-            subkeys[bfh(pubkey)] = base_path + pack('<II', aa, bb)
-            
-        for txin in inputs:
-            if txin['type'] == 'coinbase':
-                self.give_error("Coinbase not supported")
-
-            if txin['type'] in ['p2sh', 'p2wsh-p2sh', 'p2wsh']:
-                self.give_error('No support yet for inputs of type: ' + txin['type'])
-
-        # Construct PSBT from start to finish.
-        out_fd = io.BytesIO()
-        out_fd.write(b'psbt\xff')
-
-        def write_kv(ktype, val, key=b''):
-            # serialize helper: write w/ size and key byte
-            out_fd.write(my_var_int(1 + len(key)))
-            out_fd.write(bytes([ktype]) + key)
-
-            if isinstance(val, str):
-                val = bfh(val)
-
-            out_fd.write(my_var_int(len(val)))
-            out_fd.write(val)
-
-
-        # global section: just the unsigned txn
-        class CustomTXSerialization(Transaction):
-            @classmethod
-            def input_script(cls, txin, estimate_size=False):
-                return ''
-
-        unsigned = bfh(CustomTXSerialization(tx.serialize()).serialize_to_network(witness=False))
-        write_kv(PSBT_GLOBAL_UNSIGNED_TX, unsigned)
-
-        # end globals section
-        out_fd.write(b'\x00')
-
-        # inputs section
-        for txin in inputs:
-            if Transaction.is_segwit_input(txin):
-                utxo = txin['prev_tx'].outputs()[txin['prevout_n']]
-                spendable = txin['prev_tx'].serialize_output(utxo)
-                write_kv(PSBT_IN_WITNESS_UTXO, spendable)
-            else:
-                write_kv(PSBT_IN_NON_WITNESS_UTXO, str(txin['prev_tx']))
-
-            pubkeys, x_pubkeys = tx.get_sorted_pubkeys(txin)
-
-            pubkeys = [bfh(k) for k in pubkeys]
-
-            for k in pubkeys:
-                write_kv(PSBT_IN_BIP32_DERIVATION, subkeys[k], k)
-
-                if txin['type'] == 'p2wpkh-p2sh':
-                    assert len(pubkeys) == 1, 'can be only one redeem script per input'
-                    pa = hash_160(k)
-                    assert len(pa) == 20
-                    write_kv(PSBT_IN_REDEEM_SCRIPT, b'\x00\x14'+pa)
-
-            out_fd.write(b'\x00')
-
-        # outputs section
-        for o in tx.outputs():
-            # can be empty, but must be present, and helpful to show change inputs
-            # wallet.add_hw_info() adds some data about change outputs into tx.output_info
-            if o.address in tx.output_info:
-                # this address "is_mine" but might not be change (I like to sent to myself)
-                output_info = tx.output_info.get(o.address)
-                index, xpubs = output_info.address_index, output_info.sorted_xpubs
-
-                if index[0] == 1 and len(index) == 2:
-                    # it is a change output (based on our standard derivation path)
-                    assert len(xpubs) == 1      # not expecting multisig
-                    xpubkey = xpubs[0]
-
-                    # document its bip32 derivation in output section
-                    aa, bb = index
-                    assert 0 <= aa < 0x80000000
-                    assert 0 <= bb < 0x80000000
-
-                    deriv = base_path + pack('<II', aa, bb)
-                    pubkey = bfh(self.get_pubkey_from_xpub(xpubkey, index))
-
-                    write_kv(PSBT_OUT_BIP32_DERIVATION, deriv, pubkey)
-
-                    if output_info.script_type == 'p2wpkh-p2sh':
-                        pa = hash_160(pubkey)
-                        assert len(pa) == 20
-                        write_kv(PSBT_OUT_REDEEM_SCRIPT, b'\x00\x14' + pa)
-
-            out_fd.write(b'\x00')
-
-        return out_fd.getvalue()
-
-
     @wrap_busy
     def sign_transaction(self, tx, password):
-        # Build a PSBT in memory, upload it for signing.
+        # Upload PSBT for signing.
         # - we can also work offline (without paired device present)
         if tx.is_complete():
             return
 
         client = self.get_client()
 
-        assert client.dev.master_fingerprint == self.ckcc_xfp
+        assert client.dev.master_fingerprint == self.get_xfp_int()
 
-        raw_psbt = self.build_psbt(tx)
-
-        #open('debug.psbt', 'wb').write(out_fd.getvalue())
+        raw_psbt = tx.serialize_as_bytes()
 
         try:
             try:
                 self.handler.show_message("Authorize Transaction...")
 
-                client.sign_transaction_start(raw_psbt, True)
+                client.sign_transaction_start(raw_psbt)
 
                 while 1:
                     # How to kill some time, without locking UI?
@@ -545,7 +393,7 @@ class Coldcard_KeyStore(Hardware_KeyStore):
                 rlen, rsha = resp
             
                 # download the resulting txn.
-                new_raw = client.download_file(rlen, rsha)
+                raw_resp = client.download_file(rlen, rsha)
 
             finally:
                 self.handler.finished()
@@ -559,8 +407,11 @@ class Coldcard_KeyStore(Hardware_KeyStore):
             self.give_error(e, True)
             return
 
-        # trust the coldcard to re-searilize final product right?
-        tx.update(bh2u(new_raw))
+        tx2 = PartialTransaction.from_raw_psbt(raw_resp)
+        # apply partial signatures back into txn
+        tx.combine_with_other_psbt(tx2)
+        # caller's logic looks at tx now and if it's sufficiently signed,
+        # will send it if that's the user's intent.
 
     @staticmethod
     def _encode_txin_type(txin_type):
@@ -576,7 +427,7 @@ class Coldcard_KeyStore(Hardware_KeyStore):
     @wrap_busy
     def show_address(self, sequence, txin_type):
         client = self.get_client()
-        address_path = self.get_derivation()[2:] + "/%d/%d"%sequence
+        address_path = self.get_derivation_prefix()[2:] + "/%d/%d"%sequence
         addr_fmt = self._encode_txin_type(txin_type)
         try:
             try:
@@ -593,20 +444,39 @@ class Coldcard_KeyStore(Hardware_KeyStore):
             self.logger.exception('')
             self.handler.show_error(exc)
 
+    @wrap_busy
+    def show_p2sh_address(self, M, script, xfp_paths, txin_type):
+        client = self.get_client()
+        addr_fmt = self._encode_txin_type(txin_type)
+        try:
+            try:
+                self.handler.show_message(_("Showing address ..."))
+                dev_addr = client.show_p2sh_address(M, xfp_paths, script, addr_fmt=addr_fmt)
+                # we could double check address here
+            finally:
+                self.handler.finished()
+        except CCProtoError as exc:
+            self.logger.exception('Error showing address')
+            self.handler.show_error('{}.\n{}\n\n{}'.format(
+                _('Error showing address'),
+                _('Make sure you have imported the correct wallet description '
+                  'file on the device for this multisig wallet.'),
+                str(exc)))
+        except BaseException as exc:
+            self.logger.exception('')
+            self.handler.show_error(exc)
 
 
 class ColdcardPlugin(HW_PluginBase):
     keystore_class = Coldcard_KeyStore
-    minimum_library = (0, 7, 2)
-    client = None
+    minimum_library = (0, 7, 7)
 
     DEVICE_IDS = [
         (COINKITE_VID, CKCC_PID),
         (COINKITE_VID, CKCC_SIMULATED_PID)
     ]
 
-    #SUPPORTED_XTYPES = ('standard', 'p2wpkh-p2sh', 'p2wpkh', 'p2wsh-p2sh', 'p2wsh')
-    SUPPORTED_XTYPES = ('standard', 'p2wpkh', 'p2wpkh-p2sh')
+    SUPPORTED_XTYPES = ('standard', 'p2wpkh-p2sh', 'p2wpkh', 'p2wsh-p2sh', 'p2wsh')
 
     def __init__(self, parent, config, name):
         HW_PluginBase.__init__(self, parent, config, name)
@@ -615,7 +485,7 @@ class ColdcardPlugin(HW_PluginBase):
         if not self.libraries_available:
             return
 
-        self.device_manager().register_devices(self.DEVICE_IDS)
+        self.device_manager().register_devices(self.DEVICE_IDS, plugin=self)
         self.device_manager().register_enumerate_func(self.detect_simulator)
 
     def get_library_version(self):
@@ -644,6 +514,7 @@ class ColdcardPlugin(HW_PluginBase):
 
         return []
 
+    @runs_in_hwd_thread
     def create_client(self, device, handler):
         if handler:
             self.handler = handler
@@ -653,60 +524,121 @@ class ColdcardPlugin(HW_PluginBase):
         # the 'path' is unabiguous, so we'll use that.
         try:
             rv = CKCCClient(self, handler, device.path,
-                    is_simulator=(device.product_key[1] == CKCC_SIMULATED_PID))
+                            is_simulator=(device.product_key[1] == CKCC_SIMULATED_PID))
             return rv
-        except:
-            self.logger.info('late failure connecting to device?')
+        except Exception as e:
+            self.logger.exception('late failure connecting to device?')
             return None
 
     def setup_device(self, device_info, wizard, purpose):
-        devmgr = self.device_manager()
         device_id = device_info.device.id_
-        client = devmgr.client_by_id(device_id)
-        if client is None:
-            raise UserFacingException(_('Failed to create a client for this device.') + '\n' +
-                                      _('Make sure it is in the correct state.'))
-        client.handler = self.create_handler(wizard)
+        client = self.scan_and_create_client_for_device(device_id=device_id, wizard=wizard)
+        return client
 
     def get_xpub(self, device_id, derivation, xtype, wizard):
         # this seems to be part of the pairing process only, not during normal ops?
         # base_wizard:on_hw_derivation
         if xtype not in self.SUPPORTED_XTYPES:
             raise ScriptTypeNotSupported(_('This type of script is not supported with {}.').format(self.device))
-        devmgr = self.device_manager()
-        client = devmgr.client_by_id(device_id)
-        client.handler = self.create_handler(wizard)
+        client = self.scan_and_create_client_for_device(device_id=device_id, wizard=wizard)
         client.ping_check()
 
         xpub = client.get_xpub(derivation, xtype)
         return xpub
 
-    def get_client(self, keystore, force_pair=True):
-        # All client interaction should not be in the main GUI thread
-        devmgr = self.device_manager()
-        handler = keystore.handler
-        with devmgr.hid_lock:
-            client = devmgr.client_for_keystore(self, handler, keystore, force_pair)
-        # returns the client for a given keystore. can use xpub
-        #if client:
-        #    client.used()
+    @runs_in_hwd_thread
+    def get_client(self, keystore, force_pair=True, *,
+                   devices=None, allow_user_interaction=True) -> Optional['CKCCClient']:
+        # Acquire a connection to the hardware device (via USB)
+        client = super().get_client(keystore, force_pair,
+                                    devices=devices,
+                                    allow_user_interaction=allow_user_interaction)
+
         if client is not None:
             client.ping_check()
+
         return client
 
-    def show_address(self, wallet, address, keystore=None):
+    @staticmethod
+    def export_ms_wallet(wallet: Multisig_Wallet, fp, name):
+        # Build the text file Coldcard needs to understand the multisig wallet
+        # it is participating in. All involved Coldcards can share same file.
+        assert isinstance(wallet, Multisig_Wallet)
+
+        print('# Exported from Electrum', file=fp)
+        print(f'Name: {name:.20s}', file=fp)
+        print(f'Policy: {wallet.m} of {wallet.n}', file=fp)
+        print(f'Format: {wallet.txin_type.upper()}' , file=fp)
+
+        xpubs = []
+        derivs = set()
+        for xpub, ks in zip(wallet.get_master_public_keys(), wallet.get_keystores()):  # type: str, KeyStoreWithMPK
+            fp_bytes, der_full = ks.get_fp_and_derivation_to_be_used_in_partial_tx(der_suffix=[], only_der_suffix=False)
+            fp_hex = fp_bytes.hex().upper()
+            der_prefix_str = bip32.convert_bip32_intpath_to_strpath(der_full)
+            xpubs.append( (fp_hex, xpub, der_prefix_str) )
+            derivs.add(der_prefix_str)
+
+        # Derivation doesn't matter too much to the Coldcard, since it
+        # uses key path data from PSBT or USB request as needed. However,
+        # if there is a clear value, provide it.
+        if len(derivs) == 1:
+            print("Derivation: " + derivs.pop(), file=fp)
+
+        print('', file=fp)
+
+        assert len(xpubs) == wallet.n
+        for xfp, xpub, der_prefix in xpubs:
+            if derivs:
+                # show as a comment if unclear
+                print(f'# derivation: {der_prefix}', file=fp)
+
+            print(f'{xfp}: {xpub}\n', file=fp)
+
+    def show_address(self, wallet, address, keystore: 'Coldcard_KeyStore' = None):
         if keystore is None:
             keystore = wallet.get_keystore()
         if not self.show_address_helper(wallet, address, keystore):
             return
 
+        txin_type = wallet.get_txin_type(address)
+
         # Standard_Wallet => not multisig, must be bip32
-        if type(wallet) is not Standard_Wallet:
+        if type(wallet) is Standard_Wallet:
+            sequence = wallet.get_address_index(address)
+            keystore.show_address(sequence, txin_type)
+        elif type(wallet) is Multisig_Wallet:
+            assert isinstance(wallet, Multisig_Wallet)  # only here for type-hints in IDE
+            # More involved for P2SH/P2WSH addresses: need M, and all public keys, and their
+            # derivation paths. Must construct script, and track fingerprints+paths for
+            # all those keys
+
+            pubkey_deriv_info = wallet.get_public_keys_with_deriv_info(address)
+            pubkey_hexes = sorted([pk.hex() for pk in list(pubkey_deriv_info)])
+            xfp_paths = []
+            for pubkey_hex in pubkey_hexes:
+                pubkey = bytes.fromhex(pubkey_hex)
+                ks, der_suffix = pubkey_deriv_info[pubkey]
+                fp_bytes, der_full = ks.get_fp_and_derivation_to_be_used_in_partial_tx(der_suffix, only_der_suffix=False)
+                xfp_int = xfp_int_from_xfp_bytes(fp_bytes)
+                xfp_paths.append([xfp_int] + list(der_full))
+
+            script = bfh(wallet.pubkeys_to_scriptcode(pubkey_hexes))
+
+            keystore.show_p2sh_address(wallet.m, script, xfp_paths, txin_type)
+
+        else:
             keystore.handler.show_error(_('This function is only available for standard wallets when using {}.').format(self.device))
             return
 
-        sequence = wallet.get_address_index(address)
-        txin_type = wallet.get_txin_type(address)
-        keystore.show_address(sequence, txin_type)
+
+def xfp_int_from_xfp_bytes(fp_bytes: bytes) -> int:
+    return int.from_bytes(fp_bytes, byteorder="little", signed=False)
+
+
+def xfp2str(xfp: int) -> str:
+    # Standardized way to show an xpub's fingerprint... it's a 4-byte string
+    # and not really an integer. Used to show as '0x%08x' but that's wrong endian.
+    return struct.pack('<I', xfp).hex().lower()
 
 # EOF
